@@ -1,5 +1,6 @@
 #include "core/encoding.h"
 
+#include "core/encoding_japanese.h"
 #include "core/encoding_korean.h"
 #include "core/encoding_simp_chinese.h"
 #include "core/encoding_trad_chinese.h"
@@ -285,6 +286,138 @@ static const letter_code HIGH_TO_UTF8_EASTERN[HIGH_CHAR_COUNT] = {
     {0xff, 2, {0xcb, 0x99}},
 };
 
+// Mix of DEFAULT and EASTERN because the translators got creative..
+static const letter_code HIGH_TO_UTF8_CZECH[HIGH_CHAR_COUNT] = {
+    {0x80, 1, {0x3f}},
+    {0x81, 1, {0x3f}},
+    {0x82, 1, {0x3f}},
+    {0x83, 1, {0x3f}},
+    {0x84, 1, {0x3f}},
+    {0x85, 1, {0x3f}},
+    {0x86, 1, {0x3f}},
+    {0x87, 1, {0x3f}},
+    {0x88, 2, {0xcb, 0x86}},
+    {0x89, 1, {0x3f}},
+    {0x8a, 1, {0x3f}},
+    {0x8b, 1, {0x3f}},
+    {0x8c, 2, {0xc4, 0x8e}, 3, {0x44, 0xcc, 0x8c}},
+    {0x8d, 1, {0x3f}},
+    {0x8e, 1, {0x3f}},
+    {0x8f, 1, {0x3f}},
+    {0x90, 1, {0x3f}},
+    {0x91, 1, {0x3f}},
+    {0x92, 1, {0x3f}},
+    {0x93, 1, {0x3f}},
+    {0x94, 1, {0x3f}},
+    {0x95, 1, {0x3f}},
+    {0x96, 1, {0x3f}},
+    {0x97, 1, {0x3f}},
+    {0x98, 1, {0x3f}},
+    {0x99, 1, {0x3f}},
+    {0x9a, 2, {0xc5, 0xa1}, 3, {0x73, 0xcc, 0x8c}},
+    {0x9b, 1, {0x3f}},
+    {0x9c, 2, {0xc5, 0x93}},
+    {0x9d, 1, {0x3f}},
+    {0x9e, 1, {0x3f}},
+    {0x9f, 1, {0x3f}},
+    {0xa0, 1, {0x3f}},
+    {0xa1, 2, {0xcb, 0x87}},
+    {0xa2, 1, {0x3f}},
+    {0xa3, 1, {0x3f}},
+    {0xa4, 1, {0x3f}},
+    {0xa5, 1, {0x3f}},
+    {0xa6, 1, {0x3f}},
+    {0xa7, 1, {0x3f}},
+    {0xa8, 1, {0x3f}},
+    {0xa9, 1, {0x3f}},
+    {0xaa, 1, {0x3f}},
+    {0xab, 1, {0x3f}},
+    {0xac, 1, {0x3f}},
+    {0xad, 1, {0x3f}},
+    {0xae, 1, {0x3f}},
+    {0xaf, 1, {0x3f}},
+    {0xb0, 2, {0xc2, 0xb0}},
+    {0xb1, 1, {0x3f}},
+    {0xb2, 1, {0x3f}},
+    {0xb3, 1, {0x3f}},
+    {0xb4, 1, {0x3f}},
+    {0xb5, 1, {0x3f}},
+    {0xb6, 1, {0x3f}},
+    {0xb7, 1, {0x3f}},
+    {0xb8, 1, {0x3f}},
+    {0xb9, 1, {0x3f}},
+    {0xba, 1, {0x3f}},
+    {0xbb, 1, {0x3f}},
+    {0xbc, 1, {0x3f}},
+    {0xbd, 1, {0x3f}},
+    {0xbe, 1, {0x3f}},
+    {0xbf, 2, {0xc2, 0xbf}},
+    {0xc0, 2, {0xc5, 0xa0}, 3, {0x53, 0xcc, 0x8c}},
+    {0xc1, 2, {0xc3, 0x81}, 3, {0x41, 0xcc, 0x81}},
+    {0xc2, 2, {0xc3, 0x82}, 3, {0x41, 0xcc, 0x82}},
+    {0xc3, 1, {0x3f}},
+    {0xc4, 2, {0xc5, 0xbd}, 3, {0x5a, 0xcc, 0x8c}},
+    {0xc5, 1, {0x3f}},
+    {0xc6, 1, {0x3f}},
+    {0xc7, 1, {0x3f}},
+    {0xc8, 2, {0xc4, 0x8c}, 3, {0x43, 0xcc, 0x8c}},
+    {0xc9, 2, {0xc3, 0x89}, 3, {0x45, 0xcc, 0x81}},
+    {0xca, 2, {0xc4, 0x9a}, 3, {0x45, 0xcc, 0x8c}},
+    {0xcb, 1, {0x3f}},
+    {0xcc, 1, {0x3f}},
+    {0xcd, 2, {0xc3, 0x8d}, 3, {0x49, 0xcc, 0x81}},
+    {0xce, 1, {0x3f}},
+    {0xcf, 1, {0x3f}},
+    {0xd0, 1, {0x3f}},
+    {0xd1, 2, {0xc5, 0x87}, 3, {0x4e, 0xcc, 0x8c}},
+    {0xd2, 1, {0x3f}},
+    {0xd3, 2, {0xc3, 0x93}, 3, {0x4f, 0xcc, 0x81}},
+    {0xd4, 2, {0xc3, 0x9d}, 3, {0x59, 0xcc, 0x81}},
+    {0xd5, 1, {0x3f}},
+    {0xd6, 2, {0xc5, 0xa4}, 3, {0x54, 0xcc, 0x8c}},
+    {0xd7, 1, {0x3f}},
+    {0xd8, 2, {0xc5, 0x98}, 3, {0x52, 0xcc, 0x8c}},
+    {0xd9, 2, {0xc5, 0xae}, 3, {0x55, 0xcc, 0x8a}},
+    {0xda, 2, {0xc3, 0x9a}, 3, {0x55, 0xcc, 0x81}},
+    {0xdb, 1, {0x3f}},
+    {0xdc, 1, {0x3f}},
+    {0xdd, 1, {0x3f}},
+    {0xde, 1, {0x3f}},
+    {0xdf, 1, {0x3f}},
+    {0xe0, 2, {0xc3, 0xa0}, 3, {0x61, 0xcc, 0x80}},
+    {0xe1, 2, {0xc3, 0xa1}, 3, {0x61, 0xcc, 0x81}},
+    {0xe2, 2, {0xc3, 0xa2}, 3, {0x61, 0xcc, 0x82}},
+    {0xe3, 1, {0x3f}},
+    {0xe4, 2, {0xc3, 0xa4}, 3, {0x61, 0xcc, 0x88}},
+    {0xe5, 2, {0xc5, 0xa5}, 3, {0x74, 0xcc, 0x8c}},
+    {0xe6, 2, {0xc3, 0xa6}},
+    {0xe7, 2, {0xc3, 0xbd}, 3, {0x79, 0xcc, 0x81}},
+    {0xe8, 2, {0xc4, 0x8d}, 3, {0x63, 0xcc, 0x8c}},
+    {0xe9, 2, {0xc3, 0xa9}, 3, {0x65, 0xcc, 0x81}},
+    {0xea, 1, {0x3f}},
+    {0xeb, 2, {0xc3, 0xab}, 3, {0x65, 0xcc, 0x88}},
+    {0xec, 2, {0xc4, 0x9b}, 3, {0x65, 0xcc, 0x8c}},
+    {0xed, 2, {0xc3, 0xad}, 3, {0x69, 0xcc, 0x81}},
+    {0xee, 1, {0x3f}},
+    {0xef, 2, {0xc4, 0x8f}, 3, {0x64, 0xcc, 0x8c}},
+    {0xf0, 1, {0x3f}},
+    {0xf1, 2, {0xc5, 0xbe}, 3, {0x7a, 0xcc, 0x8c}},
+    {0xf2, 2, {0xc5, 0x88}, 3, {0x6e, 0xcc, 0x8c}},
+    {0xf3, 2, {0xc3, 0xb3}, 3, {0x6f, 0xcc, 0x81}},
+    {0xf4, 1, {0x3f}},
+    {0xf5, 1, {0x3f}},
+    {0xf6, 2, {0xc3, 0xb6}, 3, {0x6f, 0xcc, 0x88}},
+    {0xf7, 1, {0x3f}},
+    {0xf8, 2, {0xc5, 0x99}, 3, {0x72, 0xcc, 0x8c}},
+    {0xf9, 2, {0xc5, 0xaf}, 3, {0x75, 0xcc, 0x8a}},
+    {0xfa, 2, {0xc3, 0xba}, 3, {0x75, 0xcc, 0x81}},
+    {0xfb, 1, {0x3f}},
+    {0xfc, 2, {0xc3, 0xbc}, 3, {0x75, 0xcc, 0x88}},
+    {0xfd, 1, {0x3f}},
+    {0xfe, 1, {0x3f}},
+    {0xff, 1, {0x3f}},
+};
+
 static const letter_code HIGH_TO_UTF8_CYRILLIC[HIGH_CHAR_COUNT] = {
     {0x80, 2, {0xd0, 0x82}},
     {0x81, 2, {0xd0, 0x83}},
@@ -416,6 +549,137 @@ static const letter_code HIGH_TO_UTF8_CYRILLIC[HIGH_CHAR_COUNT] = {
     {0xff, 2, {0xd1, 0x8f}},
 };
 
+static const letter_code HIGH_TO_UTF8_GREEK[HIGH_CHAR_COUNT] = {
+    {0x80, 3, {0xe2, 0x82, 0xac}},
+    {0x81, 1, {0x3f}},
+    {0x82, 3, {0xe2, 0x80, 0x9a}},
+    {0x83, 2, {0xc6, 0x92}},
+    {0x84, 3, {0xe2, 0x80, 0x9e}},
+    {0x85, 3, {0xe2, 0x80, 0xa6}},
+    {0x86, 3, {0xe2, 0x80, 0xa0}},
+    {0x87, 3, {0xe2, 0x80, 0xa1}},
+    {0x88, 1, {0x3f}},
+    {0x89, 3, {0xe2, 0x80, 0xb0}},
+    {0x8a, 1, {0x3f}},
+    {0x8b, 3, {0xe2, 0x80, 0xb9}},
+    {0x8c, 1, {0x3f}},
+    {0x8d, 1, {0x3f}},
+    {0x8e, 1, {0x3f}},
+    {0x8f, 1, {0x3f}},
+    {0x90, 1, {0x3f}},
+    {0x91, 3, {0xe2, 0x80, 0x98}},
+    {0x92, 3, {0xe2, 0x80, 0x99}},
+    {0x93, 3, {0xe2, 0x80, 0x9c}},
+    {0x94, 3, {0xe2, 0x80, 0x9d}},
+    {0x95, 3, {0xe2, 0x80, 0xa2}},
+    {0x96, 3, {0xe2, 0x80, 0x93}},
+    {0x97, 3, {0xe2, 0x80, 0x94}},
+    {0x98, 1, {0x3f}},
+    {0x99, 3, {0xe2, 0x84, 0xa2}},
+    {0x9a, 1, {0x3f}},
+    {0x9b, 3, {0xe2, 0x80, 0xba}},
+    {0x9c, 1, {0x3f}},
+    {0x9d, 1, {0x3f}},
+    {0x9e, 1, {0x3f}},
+    {0x9f, 1, {0x3f}},
+    {0xa0, 2, {0xc2, 0xa0}},
+    {0xa1, 2, {0xce, 0x85}},
+    {0xa2, 2, {0xce, 0x86}},
+    {0xa3, 2, {0xc2, 0xa3}},
+    {0xa4, 2, {0xc2, 0xa4}},
+    {0xa5, 2, {0xc2, 0xa5}},
+    {0xa6, 2, {0xc2, 0xa6}},
+    {0xa7, 2, {0xc2, 0xa7}},
+    {0xa8, 2, {0xc2, 0xa8}},
+    {0xa9, 2, {0xc2, 0xa9}},
+    {0xaa, 1, {0x3f}},
+    {0xab, 2, {0xc2, 0xab}},
+    {0xac, 2, {0xc2, 0xac}},
+    {0xad, 2, {0xc2, 0xad}},
+    {0xae, 2, {0xc2, 0xae}},
+    {0xaf, 3, {0xe2, 0x80, 0x95}},
+    {0xb0, 2, {0xc2, 0xb0}},
+    {0xb1, 2, {0xc2, 0xb1}},
+    {0xb2, 2, {0xc2, 0xb2}},
+    {0xb3, 2, {0xc2, 0xb3}},
+    {0xb4, 2, {0xce, 0x84}},
+    {0xb5, 2, {0xc2, 0xb5}},
+    {0xb6, 2, {0xc2, 0xb6}},
+    {0xb7, 2, {0xc2, 0xb7}},
+    {0xb8, 2, {0xce, 0x88}},
+    {0xb9, 3, {0xce, 0x89}},
+    {0xba, 2, {0xce, 0x8a}},
+    {0xbb, 2, {0xc2, 0xbb}},
+    {0xbc, 2, {0xce, 0x8c}},
+    {0xbd, 2, {0xc2, 0xbd}},
+    {0xbe, 2, {0xce, 0x8e}},
+    {0xbf, 2, {0xce, 0x8f}},
+    {0xc0, 2, {0xce, 0x90}},
+    {0xc1, 2, {0xce, 0x91}},
+    {0xc2, 2, {0xce, 0x92}},
+    {0xc3, 2, {0xce, 0x93}},
+    {0xc4, 2, {0xce, 0x94}},
+    {0xc5, 2, {0xce, 0x95}},
+    {0xc6, 2, {0xce, 0x96}},
+    {0xc7, 2, {0xce, 0x97}},
+    {0xc8, 2, {0xce, 0x98}},
+    {0xc9, 2, {0xce, 0x99}},
+    {0xca, 2, {0xce, 0x9a}},
+    {0xcb, 2, {0xce, 0x9b}},
+    {0xcc, 2, {0xce, 0x9c}},
+    {0xcd, 2, {0xce, 0x9d}},
+    {0xce, 2, {0xce, 0x9e}},
+    {0xcf, 2, {0xce, 0x9f}},
+    {0xd0, 2, {0xce, 0xa0}},
+    {0xd1, 2, {0xce, 0xa1}},
+    {0xd2, 1, {0x3f}},
+    {0xd3, 2, {0xce, 0xa3}},
+    {0xd4, 2, {0xce, 0xa4}},
+    {0xd5, 2, {0xce, 0xa5}},
+    {0xd6, 2, {0xce, 0xa6}},
+    {0xd7, 2, {0xce, 0xa7}},
+    {0xd8, 2, {0xce, 0xa8}},
+    {0xd9, 2, {0xce, 0xa9}},
+    {0xda, 2, {0xce, 0xaa}},
+    {0xdb, 2, {0xce, 0xab}},
+    {0xdc, 2, {0xce, 0xac}},
+    {0xdd, 2, {0xce, 0xad}},
+    {0xde, 2, {0xce, 0xae}},
+    {0xdf, 2, {0xce, 0xaf}},
+    {0xe0, 2, {0xce, 0xb0}},
+    {0xe1, 2, {0xce, 0xb1}},
+    {0xe2, 2, {0xce, 0xb2}},
+    {0xe3, 2, {0xce, 0xb3}},
+    {0xe4, 2, {0xce, 0xb4}},
+    {0xe5, 2, {0xce, 0xb5}},
+    {0xe6, 2, {0xce, 0xb6}},
+    {0xe7, 2, {0xce, 0xb7}},
+    {0xe8, 2, {0xce, 0xb8}},
+    {0xe9, 2, {0xce, 0xb9}},
+    {0xea, 2, {0xce, 0xba}},
+    {0xeb, 2, {0xce, 0xbb}},
+    {0xec, 2, {0xce, 0xbc}},
+    {0xed, 2, {0xce, 0xbd}},
+    {0xee, 2, {0xce, 0xbe}},
+    {0xef, 2, {0xce, 0xbf}},
+    {0xf0, 2, {0xcf, 0x80}},
+    {0xf1, 2, {0xcf, 0x81}},
+    {0xf2, 2, {0xcf, 0x82}},
+    {0xf3, 2, {0xcf, 0x83}},
+    {0xf4, 2, {0xcf, 0x84}},
+    {0xf5, 2, {0xcf, 0x85}},
+    {0xf6, 2, {0xcf, 0x86}},
+    {0xf7, 2, {0xcf, 0x87}},
+    {0xf8, 2, {0xcf, 0x88}},
+    {0xf9, 2, {0xcf, 0x89}},
+    {0xfa, 2, {0xcf, 0x8a}},
+    {0xfb, 2, {0xcf, 0x8b}},
+    {0xfc, 2, {0xcf, 0x8c}},
+    {0xfd, 2, {0xcf, 0x8d}},
+    {0xfe, 2, {0xcf, 0x8e}},
+    {0xff, 1, {0x3f}},
+};
+
 static struct {
     encoding_type encoding;
     const letter_code *to_utf8_table;
@@ -582,14 +846,22 @@ encoding_type encoding_determine(language_type language)
     // Determine encoding based on language:
     // - Windows-1250 (Central/Eastern Europe) is used in Polish only
     // - Windows-1251 (Cyrillic) is used in Russian only
+    // - Windows-1253 (Greek) is used in Greek only
     // - Windows-950 (Big5) is used in Traditional Chinese only
+    // - Combination of Windows-1250/1252 is used in Czech fan translation
     // - Windows-1252 (Western Europe) is used in all other languages
     if (language == LANGUAGE_POLISH) {
         data.to_utf8_table = HIGH_TO_UTF8_EASTERN;
         data.encoding = ENCODING_EASTERN_EUROPE;
+    } else if (language == LANGUAGE_CZECH) {
+        data.to_utf8_table = HIGH_TO_UTF8_CZECH;
+        data.encoding= ENCODING_CZECH;
     } else if (language == LANGUAGE_RUSSIAN) {
         data.to_utf8_table = HIGH_TO_UTF8_CYRILLIC;
         data.encoding = ENCODING_CYRILLIC;
+    } else if (language == LANGUAGE_GREEK) {
+        data.to_utf8_table = HIGH_TO_UTF8_GREEK;
+        data.encoding = ENCODING_GREEK;
     } else if (language == LANGUAGE_TRADITIONAL_CHINESE) {
         encoding_trad_chinese_init();
         data.to_utf8_table = NULL;
@@ -602,6 +874,10 @@ encoding_type encoding_determine(language_type language)
         encoding_korean_init();
         data.to_utf8_table = NULL;
         data.encoding = ENCODING_KOREAN;
+    } else if (language == LANGUAGE_JAPANESE) {
+        encoding_japanese_init();
+        data.to_utf8_table = NULL;
+        data.encoding = ENCODING_JAPANESE;
     } else { // assume Western encoding
         data.to_utf8_table = HIGH_TO_UTF8_DEFAULT;
         data.encoding = ENCODING_WESTERN_EUROPE;
@@ -649,6 +925,8 @@ void encoding_to_utf8(const uint8_t *input, char *output, int output_length, int
             encoding_trad_chinese_to_utf8(input, output, output_length);
         } else if (data.encoding == ENCODING_SIMPLIFIED_CHINESE) {
             encoding_simp_chinese_to_utf8(input, output, output_length);
+        } else if (data.encoding == ENCODING_JAPANESE) {
+            encoding_japanese_to_utf8(input, output, output_length);
         } else {
             *output = 0;
         }
@@ -700,6 +978,9 @@ void encoding_from_utf8(const char *input, uint8_t *output, int output_length)
         } else if (data.encoding == ENCODING_SIMPLIFIED_CHINESE) {
             encoding_simp_chinese_from_utf8(input, output, output_length);
             return;
+        } else if (data.encoding == ENCODING_JAPANESE) {
+            encoding_japanese_from_utf8(input, output, output_length);
+            return;
         }
     }
 
