@@ -13,6 +13,7 @@ typedef enum {
     LANGUAGE_ITALIAN,
     LANGUAGE_SPANISH,
     // Translations done by other publishers
+    LANGUAGE_JAPANESE,
     LANGUAGE_KOREAN,
     LANGUAGE_POLISH,
     LANGUAGE_PORTUGUESE,
@@ -20,6 +21,9 @@ typedef enum {
     LANGUAGE_SWEDISH,
     LANGUAGE_SIMPLIFIED_CHINESE,
     LANGUAGE_TRADITIONAL_CHINESE,
+    // Fan translations
+    LANGUAGE_CZECH,
+    LANGUAGE_GREEK,
     LANGUAGE_MAX_ITEMS
 } language_type;
 
@@ -41,6 +45,12 @@ int locale_year_before_ad(void);
  */
 int locale_translate_money_dn(void);
 
+/**
+ * Returns the indent size to use for paragraphs in rich text
+ * @return Indent in pixels
+ */
+int locale_paragraph_indent(void);
+
 /**
  * Returns whether we should use localized filenames for the "Citizen.sav", etc autosaves
  * @return Boolean true if the filenames should be translated, false if we should use English
