@@ -28,6 +28,7 @@
             TERRAIN_ROAD | TERRAIN_BUILDING | TERRAIN_GARDEN)
 
 static int aqueduct_include_construction = 0;
+static int elevation_recalculate_trees = 0;
 
 static int is_clear(int x, int y, int size, int disallowed_terrain, int check_image)
 {
@@ -1105,7 +1106,9 @@ static void set_elevation_image(int x, int y, int grid_offset)
                 if (terrain & TERRAIN_SHRUB) {
                     map_image_set(grid_offset, image_group(GROUP_TERRAIN_SHRUB) + (map_random_get(grid_offset) & 7));
                 } else if (terrain & TERRAIN_TREE) {
-                    map_image_set(grid_offset, image_group(GROUP_TERRAIN_TREE) + (map_random_get(grid_offset) & 7));
+                    if (elevation_recalculate_trees) {
+                        update_tree_image(x, y, grid_offset);
+                    }
                 } else if (terrain & TERRAIN_ROAD) {
                     map_tiles_set_road(x, y);
                 } else if (terrain & TERRAIN_AQUEDUCT) {
@@ -1124,14 +1127,25 @@ static void set_elevation_image(int x, int y, int grid_offset)
     }
 }
 
-void map_tiles_update_all_elevation(void)
+static void update_all_elevation(int recalculate_trees)
 {
+    elevation_recalculate_trees = recalculate_trees;
     int width = map_data.width - 2;
     int height = map_data.height - 2;
     foreach_region_tile(0, 0, width, height, clear_access_ramp_image);
     foreach_region_tile(0, 0, width, height, set_elevation_image);
 }
 
+void map_tiles_update_all_elevation(void)
+{
+    update_all_elevation(0);
+}
+
+void map_tiles_update_all_elevation_editor(void)
+{
+    update_all_elevation(1);
+}
+
 void map_tiles_add_entry_exit_flags(void)
 {
     int entry_orientation;
