@@ -14,6 +14,7 @@
 #include "input/touch.h"
 #include "platform/arguments.h"
 #include "platform/file_manager.h"
+#include "platform/file_manager_cache.h"
 #include "platform/joystick.h"
 #include "platform/keyboard_input.h"
 #include "platform/platform.h"
@@ -29,14 +30,15 @@
 
 #include "platform/android/android.h"
 #include "platform/emscripten/emscripten.h"
+#include "platform/ios/ios.h"
 #include "platform/switch/switch.h"
 #include "platform/vita/vita.h"
 
-#if defined(_WIN32)
+#if defined(_WIN32) || defined(__vita__) || defined(__SWITCH__) || defined(__ANDROID__)
 #include <string.h>
 #endif
 
-#if defined(USE_TINYFILEDIALOGS) || defined(__ANDROID__)
+#if defined(USE_TINYFILEDIALOGS) || defined(__ANDROID__) || defined(__IPHONEOS__)
 #define SHOW_FOLDER_SELECT_DIALOG
 #endif
 
@@ -59,7 +61,7 @@ enum {
 static struct {
     int active;
     int quit;
-} data = {1, 0};
+} data = { 1, 0 };
 
 static void exit_with_status(int status)
 {
@@ -69,6 +71,11 @@ static void exit_with_status(int status)
     exit(status);
 }
 
+#ifdef __IPHONEOS__
+static julius_args args;
+static void setup(const julius_args *args);
+#endif
+
 static void handler(int sig)
 {
     SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Oops, crashed with signal %d :(", sig);
@@ -231,6 +238,12 @@ static void handle_window_event(SDL_WindowEvent *event, int *window_active)
         case SDL_WINDOWEVENT_LEAVE:
             mouse_set_inside_window(0);
             break;
+        case SDL_WINDOWEVENT_FOCUS_LOST:
+            mouse_set_window_focus(0);
+            break;
+        case SDL_WINDOWEVENT_FOCUS_GAINED:
+            mouse_set_window_focus(1);
+            break;
         case SDL_WINDOWEVENT_SIZE_CHANGED:
             SDL_Log("Window resized to %d x %d", (int) event->data1, (int) event->data2);
             platform_screen_resize(event->data1, event->data2);
@@ -244,11 +257,14 @@ static void handle_window_event(SDL_WindowEvent *event, int *window_active)
             break;
 
         case SDL_WINDOWEVENT_SHOWN:
-            SDL_Log("Window %d shown", (unsigned int) event->windowID);
+            SDL_Log("Window %u shown", (unsigned int) event->windowID);
+#ifdef USE_FILE_CACHE
+            platform_file_manager_cache_invalidate();
+#endif
             *window_active = 1;
             break;
         case SDL_WINDOWEVENT_HIDDEN:
-            SDL_Log("Window %d hidden", (unsigned int) event->windowID);
+            SDL_Log("Window %u hidden", (unsigned int) event->windowID);
             *window_active = 0;
             break;
     }
@@ -352,6 +368,11 @@ static void teardown(void)
     platform_screen_destroy();
     SDL_Quit();
     teardown_logging();
+    
+#ifdef __IPHONEOS__
+    // iOS apps are not allowed to self-terminate. To avoid being stuck on a blank screen here, we start the game again.
+    setup(&args);
+#endif
 }
 
 static void main_loop(void)
@@ -413,7 +434,7 @@ static int init_sdl(void)
 #ifdef SHOW_FOLDER_SELECT_DIALOG
 static const char *ask_for_data_dir(int again)
 {
-#ifdef __ANDROID__
+#if defined __ANDROID__
     if (again) {
         const SDL_MessageBoxButtonData buttons[] = {
            {SDL_MESSAGEBOX_BUTTON_RETURNKEY_DEFAULT, 1, "OK"},
@@ -434,6 +455,26 @@ static const char *ask_for_data_dir(int again)
         }
     }
     return android_show_c3_path_dialog(again);
+#elif defined __IPHONEOS__
+    if (again) {
+        const SDL_MessageBoxButtonData buttons[] = {
+           {SDL_MESSAGEBOX_BUTTON_RETURNKEY_DEFAULT, 1, "OK"},
+           {SDL_MESSAGEBOX_BUTTON_ESCAPEKEY_DEFAULT, 0, "Cancel"}
+        };
+        const SDL_MessageBoxData messageboxdata = {
+            SDL_MESSAGEBOX_WARNING, NULL, "Wrong folder selected",
+            "The selected folder is not a proper Caesar 3 folder.\n\n"
+            "Press OK to select another folder or Cancel to exit.",
+            SDL_arraysize(buttons), buttons, NULL
+        };
+        int result;
+        SDL_ShowMessageBox(&messageboxdata, &result);
+        if (!result) {
+            return NULL;
+        }
+    }
+    
+    return ios_show_c3_path_dialog(again);
 #else
     if (again) {
         int result = tinyfd_messageBox("Wrong folder selected",
@@ -445,7 +486,7 @@ static const char *ask_for_data_dir(int again)
             return NULL;
         }
     }
-    return tinyfd_selectFolderDialog("Please select your Caesar 3 folder", NULL);
+    return tinyfd_selectFolderDialog("Please select your Caesar 3 folder");
 #endif
 }
 #endif
@@ -456,6 +497,11 @@ static int pre_init(const char *custom_data_dir)
         SDL_Log("Loading game from %s", custom_data_dir);
         if (!platform_file_manager_set_base_path(custom_data_dir)) {
             SDL_Log("%s: directory not found", custom_data_dir);
+            SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR,
+                "Error",
+                "Julius requires the original files from Caesar 3.\n\n"
+                "Please enter the proper directory or copy the files to the selected directory.",
+                NULL);
             return 0;
         }
         return game_pre_init();
@@ -468,16 +514,24 @@ static int pre_init(const char *custom_data_dir)
 
 #if SDL_VERSION_ATLEAST(2, 0, 1)
     if (platform_sdl_version_at_least(2, 0, 1)) {
+#ifdef __IPHONEOS__
+        char *base_path = ios_get_base_path();
+#else
         char *base_path = SDL_GetBasePath();
+#endif
         if (base_path) {
             if (platform_file_manager_set_base_path(base_path)) {
                 SDL_Log("Loading game from base path %s", base_path);
                 if (game_pre_init()) {
+#ifndef __IPHONEOS__
                     SDL_free(base_path);
+#endif
                     return 1;
                 }
             }
+#ifndef __IPHONEOS__
             SDL_free(base_path);
+#endif
         }
     }
 #endif
@@ -497,19 +551,12 @@ static int pre_init(const char *custom_data_dir)
         if (platform_file_manager_set_base_path(user_dir) && game_pre_init()) {
             pref_save_data_dir(user_dir);
 #ifdef __ANDROID__
-            android_toast_message("C3 files found. Path saved.");
+            SDL_AndroidShowToast("C3 files found. Path saved.", 0, 0, 0, 0);
 #endif
             return 1;
         }
         user_dir = ask_for_data_dir(1);
     }
-#elif defined(__vita__)
-    SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR,
-        "Error",
-        "Julius requires the original files from Caesar 3.\n\n"
-        "Please add the files to:\n\n"
-        VITA_PATH_PREFIX,
-        NULL);
 #else
     SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR,
         "Julius requires the original files from Caesar 3 to run.",
@@ -533,7 +580,13 @@ static void setup(const julius_args *args)
         exit_with_status(-1);
     }
 
-    if (!pre_init(args->data_directory)) {
+#ifdef __vita__
+    const char *base_dir = VITA_PATH_PREFIX;
+#else
+    const char *base_dir = args->data_directory;
+#endif
+
+    if (!pre_init(base_dir)) {
         SDL_Log("Exiting: game pre-init failed");
         exit_with_status(1);
     }
@@ -544,6 +597,10 @@ static void setup(const julius_args *args)
         setting_set_display(0, w, h);
         SDL_Log("Forcing windowed mode with size %d x %d", w, h);
     }
+    if (args->force_fullscreen && !setting_fullscreen()) {
+        setting_set_display(1, 0, 0);
+        SDL_Log("Forcing fullscreen mode");
+    }
 
     // handle arguments
     if (args->display_scale_percentage) {
@@ -555,7 +612,7 @@ static void setup(const julius_args *args)
 
     char title[100];
     encoding_to_utf8(lang_get_string(9, 0), title, 100, 0);
-    if (!platform_screen_create(title, config_get(CONFIG_SCREEN_DISPLAY_SCALE))) {
+    if (!platform_screen_create(title, config_get(CONFIG_SCREEN_DISPLAY_SCALE), args->display_id)) {
         SDL_Log("Exiting: SDL create window failed");
         exit_with_status(-2);
     }
@@ -580,11 +637,17 @@ static void setup(const julius_args *args)
 int main(int argc, char **argv)
 {
     julius_args args;
-    platform_parse_arguments(argc, argv, &args);
+    if (!platform_parse_arguments(argc, argv, &args)) {
+#if !defined(_WIN32) && !defined(__vita__) && !defined(__SWITCH__) && !defined(__ANDROID__) && !defined(__APPLE__)
+        // Only exit on Linux platforms where we know the system will not throw any weird arguments our way
+        exit_with_status(1);
+#endif
+    }
 
     setup(&args);
 
     mouse_set_inside_window(1);
+    mouse_set_window_focus(1);
     run_and_draw();
 
 #ifdef __EMSCRIPTEN__
