@@ -1,4 +1,9 @@
 #include "graphics/window.h"
+#include "widget/minimap.h"
+#include "window/building_info.h"
+#include "window/editor/map.h"
+#include "window/logo.h"
+#include "window/main_menu.h"
 #include "window/message_dialog.h"
 #include "window/popup_dialog.h"
 #include "window/mission_end.h"
